import urllib3.request
import urllib.parse
import certifi
import os
import requests
from requests.exceptions import HTTPError

from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import Mail, Email

from sendgrid.helpers.mail.attachment import Attachment
import base64
from sendgrid.helpers.mail.file_content import FileContent
from sendgrid.helpers.mail.file_type import FileType
from sendgrid.helpers.mail.file_name import FileName
from sendgrid.helpers.mail.disposition import Disposition
       
class NuvIoTResponse:
    """
    The NuvIoTResponse class contains fields that were returned from
    a call to a NuvIoT service that returns a list of data.  This class
    returns the following fields.  Depending on the underlying data source
    it may return a row key, partition key or both.  It is advisable to return
    both in the requests for additional data.

    The following are the parameters returned by a list data request, not all are present
    with every request
    ------------
    * rows - array of rows of data
    * nextPartitionKey - partition key to be returned to the server to receive the next page of data.
    * nextRowKey - row key to be returned to the server to receive the next page of data.
    * pageSize - total number of records per page.
    * pageIndex - for data sources that can return a page index, the page index of the data requested.
    * pageCount - for data source that can return a total page count, the total number of pages that could be returned.
    * hasMoreRecords - true if there are more records and false if not.
 
    """
    def __init__(self, result):
        self.rows = result['model']
        self.nextPartitionKey = result['nextPartitionKey']
        self.nextRowKey = result['nextRowKey']
        self.pageSize = result['pageSize']
        self.pageIndex = result['pageIndex']
        self.pageCount = result['pageCount']
        self.hasMoreRecords = result['hasMoreRecords']

class NuvIoTRequest:
    def __init__(self, path):
        """
        The NuvIoT request class can be used to make request to different data sources within NuvIoT.  These 
        are generally data streams but may also be used to return other types of data.

        The following are the parameters returned by a list data request, not all are present
        with every request
        ------------

        * path - Path to the end point that will return data, this may include parameters as specified by the REST request specification for the data source.
        * pageSize - number of records per page to be returned most data sources support page size.
        * pageIndex - some data sources expect the page index to perform paging.  If a page index was returned from the list request it should be passed in as a parameter.
        * nextRowKey - some data source expect the a row key to be used to perform paging.  If a row key was returned from the list request it should be passed in as a parameter.
        * nextPartitionKey - some data source expect the a partition key to be used to perform paging.  If a partition key was returned from the list request it should be passed in as a parameter.
        * startDate - for queries that support filtering by date, an optional start date can be provided in the standard JSON format.
        * endDate - for queries that support filtering by date, an optional end date can be provided in the standard JSON format.
        * groupBy - for queries that allow grouping by a field, the name of the field to group the data.
        * groupBySize - for queries that allow grouping of data, the number of records to group data together.
    
        Parameters
        ----------
        path:
            Path used to query data.

        """
        self.path = path
        self.pageSize = 50
        self.pageIndex = 0
        self.nextRowKey = None
        self.nextPartitionKey = None
        self.startDate = None
        self.endDate = None
        self.groupBy = None
        self.groupBySize = None
                                      
def get(ctx, path, content_type = "", pageSize=50):
    """
    Make a GET request to a NuvIoT data source will return a string that includes JSON for the response.

    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    path:
        Path used to make the request, the auth and server information will be used from the ctx object.

    contentType: 
        Optional content type parameter used set the Accept HTTP header, defaults to empty.

    pageSize:
        Optional page size parameter used to make list requests, defaults to 50
    """
    if ctx.auth_type == 'user':
        headers={'Authorization': 'Bearer ' + ctx.auth_token, 'x-pagesize' : str(pageSize)}
    else:
        headers={'Authorization': 'APIKey ' + ctx.client_id + ':' + ctx.client_token, 'x-pagesize' : str(pageSize)}
       
    if(content_type != ""):
        headers['Accept'] = content_type
   
    http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
    url = ctx.url + path
    r = http.request("GET", url, headers=headers, preload_content=False)
    
    responseJSON = ''
    for chunk in r.stream(32):
        responseJSON += chunk.decode("utf-8")

    r.release_conn()
    
    if r.status > 299:
        print('Failed http call, response code: ' + str(r.status))
        print('Url: ' + url)
        print('Headers: ' + str(headers))
        print(responseJSON)
        print('--------------------------------------------------------------------------------')
        print()
        raise Exception("Http non success code %d not complete request to %s" % (r.status, path))
   

    return responseJSON

def post_json(ctx, path, json):
    """
    Make a POST request with a JSON payload to be sent to a NuvIoT endpoint.
    
    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    path:
        Path used to make the request, the auth and server information will be used from the ctx object.

    json:
        JSON object to be posted

    Returns
    -------
        Will return any JSON returned from the server, if the response code is not a success code an exception will be raised.
    """
    if ctx.auth_type == 'user':
        headers={'Authorization': 'Bearer ' + ctx.auth_token, 'Content-Type':'application/json'}
    else:
        headers={'Authorization': 'APIKey ' + ctx.client_id + ':' + ctx.client_token, 'Content-Type':'application/json'}
    
    url = ctx.url + path

    encoded_data = json.encode('utf-8')
    
    http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
    r = http.request('POST', url,
             headers=headers,
             preload_content=False,
             body=encoded_data)
    
    responseJSON = ''
    responseStatus = r.status
    for chunk in r.stream(32):
        responseJSON += chunk.decode("utf-8")
    
    r.release_conn()

    if responseStatus > 299:
        print('Failed http call, response code: ' + str(responseStatus))
        print('Url: ' + url)
        print('Headers: ' + str(headers))
        print(responseJSON)
        print('--------------------------------------------------------------------------------')
        print()
        raise Exception("Could not post JSON to %s" % url)

    return responseJSON  
  
def post_file(ctx, path, file_name):
    """
    Post a file to a NuvIoT end point.

    If the method does not raise an exception the file will be uploaded. 
    
    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    path:
        Path used to make the request, the auth and server information will be used from the ctx object.

    file:
        full path to the file including the file name and extension to be uploaded.

    """
    if ctx.auth_type == 'user':
        headers={'Authorization': 'Bearer ' + ctx.auth_token}
    else:
        headers={'Authorization': 'APIKey ' + ctx.client_id + ':' + ctx.client_token}

    url = ctx.url + path

    if(not os.path.isfile(file_name)):
        raise Exception("File %s does not exists" % file_name)    
    
    session = requests.Session()
    files = {'file': open(file_name, 'rb')}
    r = requests.post(url, headers = headers, files = files)
    if r.status_code > 299:
        print('Failed http call, response code: ' + str(r.status_code))
        print('Url: ' + url)
        print('Headers: ' + str(headers))
        print(r.text)
        print('--------------------------------------------------------------------------------')
        print()
        raise Exception("Error %d, could not upload %s to %s." % (r.status_code, file_name, path))  


def download_file(ctx, path, dest, accept = ""):
    """
    Download a file to a NuvIoT
    
    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    path:
        Path used to make the request, the auth and server information will be used from the ctx object.

    dest:
        The full path including the file name and extension of where the downloaded file should be saved.

    Returns
    -------
        Will return True if the file is download and saved locally, other wise it will return False.
    """
    if ctx.auth_type == 'user':
        headers={'Authorization': 'Bearer ' + ctx.auth_token}
    else:
        headers={'Authorization': 'APIKey ' + ctx.client_id + ':' + ctx.client_token}
       
    if(accept != ""):
        headers['Accept'] = accept
    
    url = ctx.url + path
    
    print("Downloading file: %s" % url)

    chunk_size = 65536
        
    http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
    r = http.request("GET", url, headers=headers, preload_content=False)
 
    if r.status > 299:
        print('Failed http %d url: %s' % (r.status, url))
        print('Headers: ' + str(headers))
        print('--------------------------------------------------------------------------------')
        print()
        r.release_conn()
        return False

    print('Headers: ' + str(r.headers))
    print('Headers: ' + str(r.headers["Content-Disposition"]))
    with open(dest, 'wb') as out:
        while True:
            data = r.read(65535)
            if not data:
                break
            
            out.write(data)
    r.release_conn()
    return True
  
def get_paged(ctx, rqst):
    """
    Make a GET request to a NuvIoT paged data source will return a string that includes JSON for the response.

    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    path:
        Path used to make the request, the auth and server information will be used from the ctx object.

    contentType: 
        Optional content type parameter used set the Accept HTTP header, defaults to empty.

    pageSize:
        Optional page size parameter used to make list requests, defaults to 50
    """    
    if ctx.auth_type == 'user':
        headers={'Authorization': 'Bearer ' + ctx.auth_token, 'x-pagesize' : rqst.pageSize}
    else:
        headers={'Authorization': 'APIKey ' + ctx.client_id + ':' + ctx.client_token, 'x-pagesize' : rqst.pageSize}    

    http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
    url = ctx.url + rqst.path
    r = http.request("GET", url, headers=headers, preload_content=False)
    responseJSON = ''
    for chunk in r.stream(32):
        responseJSON += chunk.decode("utf-8")

    r.release_conn()
    
    if r.status > 299:
        print('Failed http call, response code: ' + str(r.status))
        print('Url: ' + url)
        print('Headers: ' + str(headers))
        print(responseJSON)
        print('--------------------------------------------------------------------------------')
        print()
        return None
      

    return responseJSON


import urllib3.request
import urllib.parse
import json
import certifi

class AuthContext:
    """
    The AuthContext class is used to contain data about the user or client app that
    will be used to identify the program or user that is making calls to the server
    to get data or make updates.

    Any authorized calls as made from the python api expect an instance of an
    AuthContext class as a parameter.

    If you are creating an AuthContext class for a user you must call get_token
    to authorize the user id and password combination prior to making any calls.

    If you are creating an AuthContext for a client app, you must register
    that client app with the server and obtain a client_id and client token,
    you do not need to call get_token for client type AuthContext instances.
    """
    @classmethod
    def for_user(cls, appid, uid, pwd, url="https://api.nuviot.com"):
        """
        Creates an authorization context for a user with an user id and password

        Parameters
        ----------
        appid:
            Unique application id that will be sent along with the user login

        uid: 
            User Id, which is usually the email address for the user

        pwd:
            Password for the user

        url:
            Optional, url to login the account, unless this is a custom installation, the default will be https://api.nuviot.com
        """

        cls.auth_type = 'user'
        cls.appid = appid
        cls.uid = uid
        cls.pwd = pwd
        cls.url = url
        cls.auth_token = ''
        cls.auth_token_expires = ''
        cls.refresh_token = ''
        cls.refresh_expires = ''
        cls.auth_expires = ''
        cls.app_instance_id = ''
        return cls()
    
    @classmethod
    def for_client_app(cls, clientid, client_token, url="https://api.nuviot.com"):
        """
        Creates an authorization context for client id and client token as registered on NuvIoT

        Parameters
        ----------
        appid:
            Unique application id that will be sent along with the user login

        uid: 
            User Id, which is usually the email address for the user

        pwd:
            Password for the user

        url:
            Optional, url to login the account, unless this is a custom installation, the default will be https://api.nuviot.com
        """

        cls.auth_type = 'clientapp'
        cls.client_id = clientid
        cls.url = url
        cls.client_token = client_token
        return cls()
        
    def get_token(self):
        """
        Make an authentication call to the server to generate an authorization token
        this only applies for user type logins.

        When this method is ran the auth token and refresh token are stored in the 
        AuthContext class, so the the AuthContext class can be passed in to methods
        required to make authorized calls. 
        """
        if self.auth_type == 'user':
            post = {"grantType": "password",
              "appId": self.appid,
              "deviceId": "0000000000000000000000000000",
              "appInstanceId": "0000000000000000000000000000",
              "clientType": "aiclient",
              "email": self.uid,
              "userName": self.uid,
              "password": self.pwd,
              "refreshToken": None,
              "orgId": None,
              "orgName": None
            }

            data = json.dumps(post)

            http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
            headers={'Content-Type': 'application/json'}
            client_auth_uri = "%s/api/v1/auth" % self.url
            print(client_auth_uri)
            r = http.request("POST",client_auth_uri, body=data, headers=headers, preload_content=False)

            responseJSON = ''
            for chunk in r.stream(32):
                responseJSON += chunk.decode("utf-8")

            r.release_conn()
            print("REQUESTED AUTH")
            print(responseJSON)
            ro = json.loads(responseJSON)

            self.app_instance_id = ro["result"]["appInstanceId"]
            self.auth_token_expires = ro["result"]["accessTokenExpiresUTC"]
            self.refresh_expires = ro["result"]["refreshTokenExpiresUTC"]
            self.auth_token = ro["result"]["accessToken"]
            self.refresh_token = ro["result"]["refreshToken"]
        return



import json
import nuvpy.nuviot_srvc as nuviot_srvc
import nuvpy.nuviot_util as nuviot_util
from nuvpy.nuviot_srvc import NuvIoTRequest
from nuvpy.nuviot_srvc import NuvIoTResponse

def get_streams(ctx):
    """
    Load the details for the data streams that have been allocated for the
    the current organization

    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    """
    responseJSON = nuviot_srvc.get(ctx, '/clientapi/datastreams')
    if responseJSON == None:
        return

    rj = json.loads(responseJSON)
    return nuviot_util.to_item_array(rj)   

def print_streams(ctx):
    """
    Print the streams along with their ids that have been allocated for the current organization. 

    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    """
    responseJSON = nuviot_srvc.get(ctx, '/clientapi/datastreams')
    if responseJSON == None:
        return
 
    rj = json.loads(responseJSON)
    return nuviot_util.print_array("Data Stream", nuviot_util.to_item_array(rj))  

def get_stream(ctx, stream_id, device_id, page_size=1500):
    """
    Get a JSON array containing the data in a data stream for a particular device id. 

    Parameters
    ----------
    ctx:
        Context Object that defines how this method should call the server to include authentication.

    stream_id:
        Identifier of the data stream where data should be pulled.

    device_id:
        Device Id to pull data for.
    
    page_size:
        (optional) Number of records to return the default is 1500.

    """
    rqst = NuvIoTRequest('/clientapi/datastream/' + stream_id + '/data/' + device_id)
    rqst.pageSize = page_size
    responseJSON = nuviot_srvc.get_paged(ctx, rqst)
    if responseJSON == None:
        return
    
    rj = json.loads(responseJSON)
    response = NuvIoTResponse(rj)
    return response.rows


import os, sys
import errno
import json
import urllib3.request
import urllib.parse
import certifi
import requests
import nuvpy.nuviot_srvc as nuviot_srvc
import re

import datetime
import pandas as pd

def init(output_dir):
    """
    Initialize the report builder by passing in the directory where the output files 
    will be saved.

    Parameters
    ----------

    output_dir:
        directory where output files will be saved 

    """
    try:
        os.makedirs(output_dir)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

def add_generated_report_header(report_header):
    """
    Upload report history and return the id of the header that was generated
    on the server.

    Parameters
    ----------
    
    report_header:
        Required Parmeters:
        A dictionary of parameters that will be used to describe the report that consist of:
        - report: Name of the report
        - executionTimeMS: The number of milliseconds it took to generate the report
        - scheduled: True if the report was scheduled, false if it was not
        - note: Any notes to be added to the report
        - user: An Entity Header (dictionary of id and text) of the user (which could be a system user) that requested the report.
        - contentType: Mime type of the report, generally this is application/pdf
        - fileName: name of the file (not including the path) of the report
        - reportTitle: tile of the report as it was generated
        Optional Parameters
        - reportSummary: report summary as returned from the generatred report
        - reportDate: date of for the report
        - device: An Entity Header (dictionary of id and text) of the device that this report is for, if this is provided reports for specific devices will be available in the dashboard
    
    Returns
    -------
    
    out: string
        Returns the id of the generated report that can be used to upload a report.
    
    """
    job_server = os.environ.get('JOB_SERVER_URL')
    if(job_server is None):
        raise Exception("Missing environment variable [JOB_SERVER_URL]")

    headers={'Content-Type':'application/json'}
    
    generated_report_json = json.dumps(report_header)
    url = "%s/api/generatedreport/header" % (job_server)
    
    encoded_data = generated_report_json.encode('utf-8')
    
    http = urllib3.PoolManager(cert_reqs='CERT_REQUIRED', ca_certs=certifi.where())
    r = http.request('POST', url,
             headers=headers,
             preload_content=False,
             body=encoded_data)

    responseText = ''
    responseStatus = r.status
    for chunk in r.stream(32):
        responseText += chunk.decode("utf-8")
    
    responseJSON = json.loads(responseText)

    r.release_conn()

    if responseStatus > 299:
        print('Failed http call, response code: ' + str(responseStatus))
        print('Url: ' + url)
        print(responseJSON)
        print('--------------------------------------------------------------------------------')
        print()
        raise Exception("Could not upload report header to %s" % url)

    if(responseJSON["successful"]):
        return responseJSON["result"]
    else:
        raise Exception(responseJSON["errors"][0]["message"])

def upload_report(report_id, generated_report_id, output_file):
    """
    upload a report file to the server

    Parameters
    ----------
    report_id
        The id of the report definition that was originally created for this report.

    generated_report_id
        Id returned from adding the report_header to the server

    output_file
        Full path and file of the report that was generated

    Returns
    --------
    out: string
        Returns the full URL that can be used to access this report, note that access to this PDF will be secured by authentication.

    """
    job_server = os.environ.get('JOB_SERVER_URL')
    if(job_server is None):
        raise Exception("Missing environment variable [JOB_SERVER_URL]")

    url = "%s/api/generatedreport/%s/%s/upload" % (job_server, report_id, generated_report_id)

    if(not os.path.isfile(output_file)):
        raise Exception("File %s does not exists" % output_file)    
    
    files = {'file': open(output_file, 'rb')}
    r = requests.post(url, files = files)

    print(r.text)

    responseText = r.text
    responseStatus = r.status_code
    
    responseJSON = json.loads(responseText)

    print(responseStatus)
    if responseStatus > 299:
        print('Failed http call, response code: ' + str(responseStatus))
        print('Url: ' + url)
        print(r.text)
        print('--------------------------------------------------------------------------------')
        print()
        raise Exception("Error %d, could not upload %s to %s." % (r.status_code, output_file, url))  

    if(responseJSON["successful"]):
        return responseJSON["result"]
    else:
        raise Exception(responseJSON["errors"][0]["message"])

def add_page_header(pdf, report_title, device_name, logo_file = None, date = None):
    """
    Add a report header to a PDF

    Parameters
    ----------
    pdf
        Instance of the the PDF document that is being built.
    
    report_title
        Title to be placed at the top of the report

    device_name
        Name of the device that will be added to the report header

    logo_file
        Logo file to be added to the top of the report (optional)

    date
        Date to be added to the top of the report (optional)
    """
    if(date == None):
        date = datetime.datetime.now()
   
    if(isinstance(date, datetime.date) or isinstance(date, datetime.datetime)):
        report_date = date
    else:
        p = re.compile('[0-1]?\d\/[0-3]?\d\/\d{4} \d{1,2}:\d{2}')
        if(p.match(date) != None):
            report_date = datetime.datetime.strptime(date, "%m/%d/%Y %H:%M")
        else:
            p = re.compile('^\d{4}\/[0-1]?\d\/[0-3]?\d$')
            if(p.match(date) != None):
                report_date = datetime.datetime.strptime(date, "%Y/%m/%d")
            
    if(report_date == None):
        report_date = datetime.datetime.now()

    if(logo_file != None):
        pdf.set_xy(183.,6.0)
        pdf.image(logo_file,  link='', type='', w=1586/80, h=1920/80)
    
    pdf.set_font('Arial', 'B', 24)
    pdf.set_text_color(50, 50, 50)
    pdf.set_xy(0,2.0)
    pdf.cell(w=210.0, h=40.0, align='C', txt=report_title, border=0)
    pdf.set_font('Arial', 'B', 18)

    
    pdf.set_xy(10,12.0)
    pdf.cell(w=210.0, h=40.0, align='L', txt=device_name, border=0)

    pdf.set_font('Arial', '', 10)
    pdf.set_xy(10,18.0)
    pdf.cell(w=210.0, h=40.0, align='L', txt=report_date.strftime("%b %d, %Y"), border=0)

def add_table(pdf, title, y, cols, df):
    """
    Add a table to PDF

    Parameters
    ----------
    pdf
        Instance of the PDF document that is being built.

    title
        Title to be placed at the top of the table.

    y 
        Y position to add the table to the report

    cols
        Definition of the table to include name of the data column, width and alignment 
    """
    x = 20
   
    pdf.set_font('Arial', 'B', 14)
    pdf.set_xy(20, y)
    pdf.cell(w=40.0, h=8.0, align='L', txt=title, border=0)
    
    y += 10
    totals = []
   
    for col in cols:
        colWidth = col['w']
        hdr = col['hdr']
        pdf.set_font('Arial', 'B', 12)
        pdf.set_xy(x, y)
        pdf.cell(w=colWidth, h=5.0, align='C', txt=hdr, border=0)
        x += colWidth
        totals.append(0)

    y += 5
    pdf.set_line_width(0.0)
    pdf.line(20, y, x, y) 
    y += 2

    for i, j in df.iterrows(): 
        x = 20
        idx = 0
        for col in cols:
            pdf.set_xy(x, y)
            colWidth = col['w']

            value = j[col['fld']]
            align = None
            if('align' in col):
                align = col['align']
           
            if(isinstance(value,float)):
                totals[idx] += value
                rnd = 1
                if('rnd' in col):
                    rnd = col['rnd']

                strFmt = '{0:.' + str(rnd) + 'f}'

                value = strFmt.format(value)

                if(align == None):
                    align = 'R'
            
            if(isinstance(value, pd._libs.tslibs.timestamps.Timestamp)):
                if('fmt' in col):
                    value = value.strftime(col['fmt'])

            if(align == None):
                align = 'L'

            pdf.cell(w=colWidth, h=5.0, align=align, txt=str(value) , border=0)
            x += colWidth
            idx += 1

        y += 5

    y += 1
    pdf.set_line_width(0.0)
    pdf.line(20, y, x, y) 
    y += 1
    pdf.line(20, y, x, y) 
    y += 2

    idx = 0
    x = 20
    for col in cols:
        if('total' in col):
            pdf.set_xy(x, y)
            colWidth = col['w']
            align = 'R'
            if('align' in col):
                align = col['align']

            rnd = 1
            if('rnd' in col):
                rnd = col['rnd']
    
            total_hdr = ""
            if('tlthdr' in col):
                total_hdr = col["tlthdr"] + " "

            value = totals[idx]
        
            if(col['total'] == 'avg'):
                value /= df[col['fld']].count()

            strFmt = '{0:.' + str(rnd) + 'f}'
            
            if('units' in col):
                strFmt += col['units']
            
            actual = strFmt.format(value)
           
            pdf.cell(w=colWidth, h=5.0, align=align, txt=total_hdr + str(actual) , border=0)
       
        x += col['w']    

        idx += 1
        
    return y


import os
import json
import requests
from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import Mail, Email

from sendgrid.helpers.mail.attachment import Attachment
import base64
from sendgrid.helpers.mail.file_content import FileContent
from sendgrid.helpers.mail.file_type import FileType
from sendgrid.helpers.mail.file_name import FileName
from sendgrid.helpers.mail.disposition import Disposition

reports_from_name = os.environ.get('Smtp__FromName')
reports_from_email = os.environ.get('Smtp__FromAddress')
sendgrid_api_key = os.environ.get('Smtp__Token')

job_server = os.environ.get('JOB_SERVER_URL')

def send_report_to_distribution(distribution_list_id, file_with_path, file_name, msg_subject, msg_content):
    """
    Sends a report/file to a distribution list as identified within NuvIot.

    Parameters
    ----------
    distribution_list_id : string
        distribution list identifier to download

    file_with_path : string
        entire path including file, used to load report output

    file_name : string
        name of the file, used to name the file as it is attached to the email

    msg_subject : string
        subject fo the message to be sent

    msg_content : string
        content to be included in the message
    """

    getJobUri = job_server + '/api/distro/' + distribution_list_id

    r = requests.get(getJobUri)
    if(r.status_code > 299):
        raise Exception("Could not get distribution list with id=%s " % (distribution_list_id))

    distro_list = json.loads(r.text)
    print(distro_list)

    for email in distro_list:
        send_report(email, file_with_path, file_name, msg_subject, msg_content)
  
def send_report(to_email_address, file_with_path, file_name, msg_subject, msg_content):
    """
    Sends a report/file to an email address

    Parameters
    ----------
    to_email_address : string
        email address to send the file

    file_with_path : string
        entire path including file, used to load report output

    file_name : string
        name of the file, used to name the file as it is attached to the email

    msg_subject : string
        subject fo the message to be sent

    msg_content : string
        content to be included in the message
    """

    message = Mail(
        from_email=Email(reports_from_email, reports_from_name),
        to_emails=to_email_address,
        subject=msg_subject,
        html_content=msg_content)

    with open(file_with_path, 'rb') as f:
        data = f.read()
        f.close()

    encoded_file = base64.b64encode(data).decode()

    message.attachment = Attachment(FileContent(encoded_file), FileName(file_name), FileType('application/pdf'), Disposition('attachment'))

    sg = SendGridAPIClient(sendgrid_api_key)
    response = sg.send(message)
    if(response.status_code >= 200 and response.status_code < 300):
        print("success sending: " + file_with_path + " to " + to_email_address + " ok")
    else:
        print("could not send email")
        print(response.status_code, response.body, response.headers)
